import random


class Spielstand():
    winning_lines = [(0, 1, 2),
                     (3, 4, 5),
                     (6, 7, 8),
                     (0, 3, 6),
                     (1, 4, 7),
                     (2, 5, 8),
                     (0, 4, 8),
                     (2, 4, 6)]
    vsep = " | "
    hsep = "-"

    alleFeldNummern = [1, 2, 3, 4, 5, 6, 7, 8, 9]

    def __init__(self):
        self.squares = [None for x in range(0, 9)]
        self.next_player = "X"
        self.gameOver = False
        self.winner = None

    def checkGameWon(self):
        for loc1, loc2, loc3 in Spielstand.winning_lines:
            if self.squares[loc1] is not None \
                    and self.squares[loc2] is not None \
                    and self.squares[loc3] is not None \
                    and self.squares[loc1] == self.squares[loc2] \
                    and self.squares[loc1] == self.squares[loc3]:
                self.winner = self.squares[loc1]
                self.gameOver = True
                return ()

    def checkGameOver(self):
        if not self.gameOver:
            self.checkGameWon()

            if not self.gameOver:
                for loc in range(0, 9):
                    if self.squares[loc] is None:
                        return ()

                self.gameOver = True

    def __repr__(self, numbered=False):
        return (self.__repr_square__(0, numbered) + self.vsep +
                self.__repr_square__(1, numbered) + self.vsep +
                self.__repr_square__(2, numbered) + "\n" +
                self.hsep * (3 + 2 * len(self.vsep)) + "\n" +
                self.__repr_square__(3, numbered) + self.vsep +
                self.__repr_square__(4, numbered) + self.vsep +
                self.__repr_square__(5, numbered) + "\n" +
                self.hsep * (3 + 2 * len(self.vsep)) + "\n" +
                self.__repr_square__(6, numbered) + self.vsep +
                self.__repr_square__(7, numbered) + self.vsep +
                self.__repr_square__(8, numbered))

    def __repr_square__(self, loc, numbered):
        if self.squares[loc] is None:
            if numbered:
                return (str(self.alleFeldNummern[loc]))
            else:
                return ("-")
        else:
            return (str(self.squares[loc]))

    def besetzeFeld(self, feldNummer):
        cloned = Spielstand()
        cloned.squares = list(self.squares)
        cloned.squares[feldNummer - 1] = self.next_player
        if self.next_player == "X":
            cloned.next_player = "O"
        else:
            cloned_next_player = "X"
        cloned.checkGameOver()
        return (cloned)

    def freieFelder(self):
        free = [nr for (nr, spieler) in zip(self.alleFeldNummern, self.squares) if spieler is None]
        random.shuffle(free)
        return (free)

    def spielBeendet(self):
        return self.gameOver

    def gewinner(self):
        if self.gameOver:
            if self.winner is None:
                return "Niemand"
            else:
                return self.winner
        else:
            return "Niemand"
    def O_hat_gewonnen(self):
        return self.winner == "O"

    def X_hat_gewonnen(self):
        return self.winner == "X"
    
    def unentschieden(self):
        return self.winner is None
    
    

    def zeigeAn(self):
        print(self)

        if self.gameOver:
            if self.winner is None:
                print("Das Spiel ist unentschieden ausgegangen")
            else:
                print("\nSpieler", self.winner, "hat gewonnen.")
        else:
            print("\nSpieler", str(self.next_player), "ist am Zug.")


def anfangszustand():
    return (Spielstand())


def playKI(zustand, ki):
    if ki is not None:
        ki_wert, ki_choice = ki(zustand)
        if ki_choice in zustand.freieFelder():
            return (zustand.besetzeFeld(ki_choice))
        print("Irgendetwas ist schief gelaufen.", end=" ")
        if ki_choice is None or ki_choice < 1 or ki_choice > 9:
            print("Die KI konnte sich für kein Feld entscheiden.")
        else:
            print("Die KI hat ein bereits besetzes Feld gewählt.")
        print("Ich probiere es einfach mal mit einem zufälligen Feld.")
    return (zustand.besetzeFeld(random.choice(zustand.freieFelder())))


def starteSpiel(ki=None):
    try:
        playFirst, firstPlayer, firstText = random.choice(
            [(True, "O", "Ich fange"),
             (False, "X", "Du fängst")])

        print("Schön, dass du eine Runde Tic-Tac-Toe spielen möchtest!")
        print("Du spielst als \"X\", ich als \"O\".", firstText, "an!")

        zustand = Spielstand()
        zustand.next_player = firstPlayer

        while not zustand.gameOver:
            if zustand.next_player == "O":
                try:
                    zustand = playKI(zustand, ki)
                except:
                    print("Fehler innerhalb der KI! Spiel wird beendet.")
                    return
            else:
                print()
                print(zustand.__repr__(True))

                user_choice = None
                while user_choice is None:
                    user_choice = input("Gib die Nummer des Feldes ein, das du besetzen möchtest: ")
                    try:
                        feld = int(user_choice)
                        if feld < 1 or feld > 9:
                            print("Es gibt kein Feld mit der Nummer ", feld,
                                  ". Probiere es noch einmal!\n", sep="")
                            user_choice = None
                        elif not feld in zustand.freieFelder():
                            print("Das Feld", feld, "ist nicht frei.",
                                  "Probiere es noch einmal!\n")
                            user_choice = None
                    except ValueError:
                        print("\"", user_choice,
                              "\" ist keine Zahl. Probiere es noch einmal!\n",
                              sep="")
                        user_choice = None
                zustand = zustand.besetzeFeld(feld)

        if zustand.gewinner() == "O":
            print("Ich habe gewonnen!!!")
        elif zustand.gewinner() == "X":
            print("Mist, du hast gewonnen!")
        else:
            print("Unentschieden!")

        print(zustand)
    except (KeyboardInterrupt, EOFError):
        print("\nSpiel wird beendet.")
